(*
 *	Copyright (C) 2003-2004 Gabest
 *	http://www.gabest.org
 *
 *  This Program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This Program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with GNU Make; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *  http://www.gnu.org/copyleft/gpl.html
 *
 *  Delphi  Translation (C) 2004 Milenko Mitrovic <dcoder@dsp-worx.de>
 *  based on the 1.4 CVS Revision of 4. Jan. 2004
 *)

unit VSFilter;

interface

uses
  Windows, ActiveX, DirectShow9, SysUtils, BaseClass;

type
  IDirectVobSub = interface(IUnknown)
    ['{EBE1FB08-3957-47ca-AF13-5827E5442E56}']
    function get_FileName(
         out fn : array of WideChar
      ) : HRESULT; stdcall; // fn should point to a buffer allocated to at least the length of MAX_PATH (=260)

    function put_FileName(
         fn : PWChar
      ) : HRESULT; stdcall;

    function get_LanguageCount(
         out nLangs : integer
      ) : HRESULT; stdcall;

    function get_LanguageName(
         iLanguage : integer;
         out ppName : PWideChar
      ) : HRESULT; stdcall; // the returned *ppName is allocated with CoTaskMemAlloc

    function get_SelectedLanguage(
         out iSelected : integer
      ) : HRESULT; stdcall;

    function put_SelectedLanguage(
         iSelected : integer
      ) : HRESULT; stdcall;

    function get_HideSubtitles(
         out fHideSubtitles : BOOL
      ) : HRESULT; stdcall;

    function put_HideSubtitles(
         fHideSubtitles : BOOL
      ) : HRESULT; stdcall;

    function get_PreBuffering(
         out fDoPreBuffering : BOOL
      ) : HRESULT; stdcall;

    function put_PreBuffering(
         fDoPreBuffering : BOOL
      ) : HRESULT; stdcall;

    function get_Placement(
         out fOverridePlacement : BOOL;
         out xperc : integer;
         out yperc : integer
      ) : HRESULT; stdcall;

    function put_Placement(
         fOverridePlacement : BOOL;
         xperc : integer;
         yperc : integer
      ) : HRESULT; stdcall;

    function get_VobSubSettings(
         out fBuffer : BOOL;
         out fOnlyShowForcedSubs : BOOL;
         out fPolygonize : BOOL
      ) : HRESULT; stdcall;

    function put_VobSubSettings(
         fBuffer : BOOL;
         fOnlyShowForcedSubs : BOOL;
         fPolygonize : BOOL
      ) : HRESULT; stdcall;

    function get_TextSettings(
         var lf; // ???
         lflen : integer; // depending on lflen, lf must point to LOGFONTA or LOGFONTW
         out color : COLORREF;
         out fShadow : BOOL;
         out fOutline : BOOL;
         out fAdvancedRenderer : BOOL
      ) : HRESULT; stdcall;

    function put_TextSettings(
         var lf;
         lflen : integer;
         color : COLORREF;
         fShadow : BOOL;
         fOutline : BOOL;
         fAdvancedRenderer : BOOL
      ) : HRESULT; stdcall;

    function get_Flip(
         out fPicture : BOOL;
         out fSubtitles : BOOL
      ) : HRESULT; stdcall;

    function put_Flip(
         fPicture : BOOL;
         fSubtitles : BOOL
      ) : HRESULT; stdcall;

    function get_OSD(
         out fOSD : BOOL
      ) : HRESULT; stdcall;

    function put_OSD(
         fOSD : BOOL
      ) : HRESULT; stdcall;

    function get_SaveFullPath(
         out fSaveFullPath : BOOL
      ) : HRESULT; stdcall;

    function put_SaveFullPath(
         fSaveFullPath : BOOL
      ) : HRESULT; stdcall;

    function get_SubtitleTiming(
         out delay : integer;
         out speedmul : integer;
         out speeddiv : integer
      ) : HRESULT; stdcall;

    function put_SubtitleTiming(
         delay : integer;
         speedmul : integer;
         speeddiv : integer
      ) : HRESULT; stdcall;

    function get_MediaFPS(
         out fEnabled : BOOL;
         out fps : Double
      ) : HRESULT; stdcall;

    function put_MediaFPS(
         fEnabled : BOOL;
         fps : Double
      ) : HRESULT; stdcall;

    function get_ColorFormat(
         out iPosition : integer
      ) : HRESULT; stdcall;

    function put_ColorFormat(
         iPosition : integer
      ) : HRESULT; stdcall;

    function get_ZoomRect(
         out rect : NORMALIZEDRECT
      ) : HRESULT; stdcall;

    function put_ZoomRect(
         rect : PNORMALIZEDRECT
      ) : HRESULT; stdcall;

    function UpdateRegistry : HRESULT; stdcall;

    function HasConfigDialog(
         iSelected : integer
      ) : HRESULT; stdcall;

    function ShowConfigDialog( // if available, this will popup a child dialog allowing the user to edit the style options
         iSelected : integer;
         hWndParent : hWnd
      ) : HRESULT; stdcall;

    function IsSubtitleReloaderLocked(
         out fLocked : BOOL
      ) : HRESULT; stdcall;

    function LockSubtitleReloader(
         fLock : BOOL
      ) : HRESULT; stdcall;

    function get_SubtitleReloader(
         out fDisabled : BOOL
      ) : HRESULT; stdcall;

    function put_SubtitleReloader(
         fDisable : BOOL
      ) : HRESULT; stdcall;

    function get_ExtendPicture(
         out horizontal : integer; // 0 - disabled, 1 - mod32 extension (width = (width+31)&~31)
         out vertical : integer;   // 0 - disabled, 1 - 16:9, 2 - 4:3, 0x80 - crop (use crop together with 16:9 or 4:3, eg 0x81 will crop to 16:9 if the picture was taller)
         out resx2 : integer;      // 0 - disabled, 1 - enabled, 2 - depends on the original resolution
         out resx2minw : integer;  // resolution doubler will be used if width*height <= resx2minw*resx2minh (resx2minw*resx2minh equals to 384*288 by default)
         out resx2minh : integer
      ) : HRESULT; stdcall;

    function put_ExtendPicture(
         horizontal : integer;
         vertical : integer;
         resx2 : integer;
         resx2minw : integer;
         resx2minh : integer
      ) : HRESULT; stdcall;

    function get_LoadSettings(
         out level : integer; // 0 - when needed, 1 - always, 2 - disabled
         out fExternalLoad : BOOL;
         out fWebLoad : BOOL;
         out fEmbeddedLoad : BOOL
      ) : HRESULT; stdcall;

    function put_LoadSettings(
         level : integer;
         fExternalLoad : BOOL;
         fWebLoad : BOOL;
         fEmbeddedLoad : BOOL
      ) : HRESULT; stdcall;
  end;

  // TODO: add STSStyle conversion
  STSStyle = Pointer;
  PSTSStyle = ^STSStyle;

  ISubClock = interface(IUnknown)
    ['{FE6EC6A0-21CA-4970-9EF0-B296F7F38AF0}']
    function SetTime (rt : TReferenceTime) : HRESULT; stdcall;
    function GetTime : TReferenceTime; stdcall;
  end;

  ISubClock2 = interface(ISubClock)
    ['{0665B760-FBC1-46C3-A35F-E471527C96A4}']
    function SetAvgTimePerFrame (rt : TReferenceTime) : HRESULT; stdcall;
    function GetAvgTimePerFrame (out prt : TReferenceTime) : HRESULT; stdcall; // return S_OK only if *prt was set and is valid
  end;

  IDirectVobSub2 = interface(IDirectVobSub)
    ['{AB52FC9C-2415-4dca-BC1C-8DCC2EAE8150}']
    function AdviseSubClock (pSubClock : ISubClock) : HRESULT; stdcall;
    function get_Forced : BOOL; stdcall;
    function put_Forced (fForced : BOOL) : HRESULT; stdcall;
    function get_TextSettings (out pDefStyle : STSStyle) : HRESULT; stdcall;
    function put_TextSettings (pDefStyle : PSTSStyle) : HRESULT; stdcall;
  end;

  TBCSubClock = class(TBCUnknown, ISubClock, ISubClock2)
    FTime : TReferenceTime;
    FTimePerFrame: TReferenceTime;
    function NonDelegatingQueryInterface(const IID: TGUID; out Obj): HResult; override;
  public
    constructor Create;
    // ISubClock
    function SetTime(rt : TReferenceTime) : HRESULT; stdcall;
    function GetTime : TReferenceTime; stdcall;
    // ISubClock2
    function SetAvgTimePerFrame (rt : TReferenceTime) : HRESULT; stdcall;
    function GetAvgTimePerFrame (out prt : TReferenceTime) : HRESULT; stdcall;
 end;

implementation

constructor TBCSubClock.Create;
begin
  inherited Create('TBCSubClock',nil);
  FTime := 0;
  FTimePerFrame := 0;
end;

function TBCSubClock.NonDelegatingQueryInterface(const IID: TGUID; out Obj): HResult;
begin
  if IsEqualGUID(IID,ISubClock) or IsEqualGUID(IID,ISubClock2) then
  begin
    if GetInterface(IID,Obj) then Result := S_OK
                             else Result := E_NOINTERFACE;
  end else Result := inherited NonDelegatingQueryInterface(IID,Obj)
end;

function TBCSubClock.SetTime(rt : TReferenceTime) : HRESULT; stdcall;
begin
  FTime := rt;
  Result := S_OK;
end;

function TBCSubClock.GetTime : TReferenceTime; stdcall;
begin
  Result := FTime;
end;

function TBCSubClock.SetAvgTimePerFrame (rt : TReferenceTime) : HRESULT; stdcall;
begin
  FTimePerFrame := rt;
  Result := S_OK;
end;

function TBCSubClock.GetAvgTimePerFrame (out prt : TReferenceTime) : HRESULT; stdcall;
begin
  if not Assigned(@prt) then
  begin
    Result := S_FALSE;
    Exit;
  end;
  prt := FTimePerFrame;
  Result := S_OK;
end;

end.
